-- DF7807
-- Specialisation for the Deutz Fahr 7807
--
-- @ Autor  Tobias F. (John Deere 6930 @ 1. Ls-Modschmiede http://landwirtschaftssimulator-mods.de/)
-- @ Last Edit  29/12/2009

DF7807 = {};

function DF7807.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Steerable, specializations);
end;

function DF7807:load(xmlFile)
	self.Go.FrontWindow = false;
	self.Done.FrontWindow = true; 
	self.Go.rearWindow = false;
	self.Done.rearWindow = true; 
	self.Go.RoofWindow = false;
	self.Done.RoofWindow = true; 

	self.Kardan = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.Kardan#index"));
	
	self.BackPuller = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.hydraulics.BackPuller#index1"));
	self.BackPullerRot = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.hydraulics.BackPuller#index2"));
	local x,y,z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.hydraulics.BackPuller#CorrectRot"));
	if (x and y and z)~= nil then
		self.BackPullerCorrectRot = {x, y, z};
	else
		self.BackPullerCorrectRot = {0, 0, 0};
	end;
	
	local hydraulicsCount = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.hydraulics#count"), 0);
	self.hydraulics = {};
	for i=1, hydraulicsCount do
		local hydraulicName = string.format("vehicle.hydraulics.hydraulic%d", i);
		self.hydraulics[i] = {};
		self.hydraulics[i].isAttacherHydr = Utils.getNoNil(getXMLBool(xmlFile, hydraulicName .. "#isAttacherHydr"), false);
		if self.hydraulics[i].isAttacherHydr then
			self.hydraulics[i].attacherIndex = getXMLString(xmlFile, hydraulicName .. "#attacherIndex");
		end;	
		self.hydraulics[i].node = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#index"));
		self.hydraulics[i].punch = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#punch"));
		self.hydraulics[i].translationPunch = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#punchFixpoint"));
		self.hydraulics[i].FixPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#fixpoint"));
		self.hydraulics[i].isScaleAble = Utils.getNoNil(getXMLBool(xmlFile, hydraulicName .. "#isScaleAble"), false);
		if self.hydraulics[i].punch ~= nil then 
			local ax, ay, az = getWorldTranslation(self.hydraulics[i].punch);
			local bx, by, bz = getWorldTranslation(self.hydraulics[i].translationPunch);
			self.hydraulics[i].punchDistance = Utils.vector3Length(ax-bx, ay-by, az-bz);
		end;
	end; 	

    self.CouplingParts = {};
    local i = 0;
    while true do
        local key = string.format("vehicle.CouplingParts.node(%d)", i);
 		local index = getXMLString(xmlFile, key.."#index");
		
        if index==nil then
            break;
        end;
		
		local entry = {};
        local node = Utils.indexToObject(self.components, index);

        entry.node = node;
		entry.onAttachState = Utils.getNoNil(getXMLBool(xmlFile, key.."#onAttachState"), false);
		local Attacherindex = getXMLInt(xmlFile, key.."#AttacherIndex");
		if Attacherindex == nil then
			break;
		end;
		entry.attacherIndex = Attacherindex;
		
		local vis = Utils.getNoNil(getXMLBool(xmlFile, key.."#setsVisible"), false);	
		if not vis then
			local rot = getXMLBool(xmlFile, key.."#setsRotation");
			entry.setsRotation = rot;
			local x,y,z = Utils.getVectorFromString(getXMLString(xmlFile, key.."#maxRot"));
			entry.maxRot = {Utils.degToRad(x), Utils.degToRad(y), Utils.degToRad(z)};		
			local x,y,z = Utils.getVectorFromString(getXMLString(xmlFile, key.."#minRot"));
			entry.minRot = {Utils.degToRad(x), Utils.degToRad(y), Utils.degToRad(z)};
			entry.rotTime = Utils.getNoNil(getXMLString(xmlFile, key.."rotTime"), 2)*1000;
			entry.isDown = onAttachState;
		end;
		entry.setsVisible = vis;		
		table.insert(self.CouplingParts, entry);
        i = i +1;
    end;
	
    self.infoHUDactive = false;
	self.infoHUDPath = Utils.getFilename(getXMLString(xmlFile, "vehicle.infoHUD#path"), self.baseDirectory);
    self.infoHUDPoxX = 0.012;
    self.infoHUDPoxY = 0.012;
    self.infoHUDWidth = 0.35;
    self.infoHUDHeight = 0.22;
    self.infoHUDOverlay = Overlay:new("infoHUDOverlay", self.infoHUDPath, self.infoHUDPoxX, self.infoHUDPoxY, self.infoHUDWidth, self.infoHUDHeight);

    self.wipers = {};
    local i = 0;
    while true do
        local key = string.format("vehicle.wipers.node(%d)", i);
 		local index = getXMLString(xmlFile, key.."#index");
		if index==nil then
            break;
        end;		
		local entry = {};
        local node = Utils.indexToObject(self.components, index);
        entry.node = node;
		
		local x,y,z = Utils.getVectorFromString(getXMLString(xmlFile, key.."#maxRot"));
		entry.maxRot = {Utils.degToRad(x), Utils.degToRad(y), Utils.degToRad(z)};		
		local x,y,z = Utils.getVectorFromString(getXMLString(xmlFile, key.."#minRot"));
		entry.minRot = {Utils.degToRad(x), Utils.degToRad(y), Utils.degToRad(z)};
		entry.rotTime = Utils.getNoNil(getXMLString(xmlFile, key.."rotTime"), 1)*1000;
	
		entry.delay1 = 0;
		entry.delay2 = 0;		
		entry.delay1reset = entry.rotTime;
		entry.delay2reset = entry.rotTime;
		
		entry.isDown = false;
		entry.isDownState = false;
			
		table.insert(self.wipers, entry);
        i = i +1;
    end;
	
    local numWorklights = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.Worklights#count"), 0);
    self.Worklights = {};
    for i=1, numWorklights do
        local lightnamei = string.format("vehicle.Worklights.light" .. "%d", i);
        self.Worklights[i] = Utils.indexToObject(self.components, getXMLInt(xmlFile, lightnamei .. "#index"));
        setVisibility(self.Worklights[i], false);
    end;
    self.WorklightsActive = false;	    
	
    self.fender = {};
    local i = 0;
    while true do
        local key = string.format("vehicle.fender.node(%d)", i);
 		local index = getXMLString(xmlFile, key.."#index");
		if index==nil then
            break;
        end;		
		local entry = {};
        local node = Utils.indexToObject(self.components, index);
        entry.node = node;			
		table.insert(self.fender, entry);
        i = i +1;
    end;	
	self.FenderActive = true;
end;

function DF7807:delete()
	if self.infoHUDOverlay ~= nil then
		delete(self.infoHUDOverlay);
	end;
end;

function DF7807:mouseEvent(posX, posY, isDown, isUp, button)
end;

function DF7807:keyEvent(unicode, sym, modifier, isDown)
end;
                                              
function DF7807:update(dt)
	if self:getIsActiveForInput() then
		if InputBinding.hasEvent(InputBinding.DF7807_INFOHUD) then
			self.infoHUDactive = not self.infoHUDactive;
		end;
		if InputBinding.hasEvent(InputBinding.DF7807_WINDOW_1) then
			self.Go.FrontWindow = not self.Go.FrontWindow;
			self.Done.FrontWindow = true; 
		end;
		if InputBinding.hasEvent(InputBinding.DF7807_WINDOW_2) then
			self.Go.rearWindow = not self.Go.rearWindow;
			self.Done.rearWindow = true; 
		end;
		if InputBinding.hasEvent(InputBinding.DF7807_WINDOW_3) then
			self.Go.RoofWindow = not self.Go.RoofWindow;
			self.Done.RoofWindow = true; 
		end;
		if InputBinding.hasEvent(InputBinding.DF7807_WORKLIGHTS) then
			self.WorklightsActive = not self.WorklightsActive;
		end;
		if InputBinding.hasEvent(InputBinding.DF7807_FENDER) then
			self.FenderActive = not self.FenderActive;
		end;
	end;
	if self:getIsActive() then
		for i=1, table.getn(self.Worklights) do
			setVisibility(self.Worklights[i], self.WorklightsActive);
		end;
		for i=1, table.getn(self.fender) do
			setVisibility(self.fender[i].node, self.FenderActive);
		end;
		
		for k, wiper in pairs(self.wipers) do
			if not (g_currentMission.environment.lastRainScale <= 0.1 and g_currentMission.environment.timeSinceLastRain > 30) then
				if wiper.delay1 >= 0 then
					wiper.delay1 = wiper.delay1 -dt;
				end;
				if wiper.delay2 >= 0 then
					wiper.delay2 = wiper.delay2 -dt;
				end;
				if not wiper.isDownState then
					if wiper.delay1 < 0 then
						wiper.delay2 = wiper.delay2reset;
						wiper.isDownState = true;
						wiper.isDown = true;
					end;
				end;
				if wiper.isDownState then
					if wiper.delay2 < 0 then
						wiper.delay1 = wiper.delay1reset;
						wiper.isDownState = false;
						wiper.isDown = false;
					end;
				end;
			else
				wiper.isDownState = false;
				wiper.isDown = false;
			end;
			local x, y, z = getRotation(wiper.node);
			local rot = {x,y,z};
			local newRot = Utils.getMovedLimitedValues(rot, wiper.maxRot, wiper.minRot, 3, wiper.rotTime, dt, not wiper.isDown);
			setRotation(wiper.node, unpack(newRot));
		end;

		if self.Kardan~=nil then
			local zRot = (self.lastSpeed*self.movingDirection*1000)/2
			rotate(self.Kardan, 0, 0, zRot);
		end;
		
		if self.Go.FrontWindow ~= nil and self.Done.FrontWindow ~= false then
			self:anim("FrontWindow", false); 
		end;	
		if self.Go.rearWindow ~= nil and self.Done.rearWindow ~= false then
			self:anim("rearWindow", false); 
		end;	
		if self.Go.RoofWindow ~= nil and self.Done.RoofWindow ~= false then
			self:anim("RoofWindow", false); 
		end;

		for k, couplingPart in pairs(self.CouplingParts) do
			for i, attacherJoint in pairs(self.attacherJoints) do
				if i == 1 then
					if attacherJoint.jointIndex ~= 0 then
						if (self.BackPuller and self.BackPullerRot) ~= nil then 
							local x, y, z = getRotation(self.BackPullerRot);
							local CR = self.BackPullerCorrectRot
							setRotation(self.BackPuller, x+CR[1], y+CR[2], z+CR[3]);
						end;
						for i, implement in pairs(self.attachedImplements) do
							if implement.jointDescIndex == 1 then
								if implement.object.attacherJoint.topReferenceNode ~= nil then
									local ax, ay, az = getWorldTranslation(attacherJoint.topArm.rotationNode);
									local bx, by, bz = getWorldTranslation(implement.object.attacherJoint.topReferenceNode);
									local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
									local scaleZ = distance/attacherJoint.topArm.referenceDistance;
									setTranslation(attacherJoint.topArm.translationNode,0,0,0);	
									setScale(attacherJoint.topArm.translationNode,1,1,scaleZ);
								end;
							end;
						end;
					else
						setScale(attacherJoint.topArm.translationNode,1,1,1);
					end;
				end;				
				if i == couplingPart.attacherIndex then	
					if couplingPart.setsVisible then
						if attacherJoint.jointIndex ~= 0 then
							setVisibility(couplingPart.node, couplingPart.onAttachState);
						else
							setVisibility(couplingPart.node, not couplingPart.onAttachState);
						end;
					elseif couplingPart.setsRotation then
						local x, y, z = getRotation(couplingPart.node);
						local rot = {x,y,z};
						local newRot = Utils.getMovedLimitedValues(rot, couplingPart.maxRot, couplingPart.minRot, 3, couplingPart.rotTime, dt, not couplingPart.isDown);
						setRotation(couplingPart.node, unpack(newRot));
						if attacherJoint.jointIndex ~= 0 then
							couplingPart.isDown = couplingPart.onAttachState;
						else
							couplingPart.isDown = not couplingPart.onAttachState;
						end;
					end;
					
				end;
			end;	
		end;
		for k, hydraulic in pairs(self.hydraulics) do
			if hydraulic.isAttacherHydr then
				for i, attacherJoint in pairs(self.attacherJoints) do
					if attacherJoint.jointIndex ~= 0 then
						local ax, ay, az = getWorldTranslation(hydraulic.node);
						local bx, by, bz = getWorldTranslation(hydraulic.FixPoint);
						local x, y, z = worldDirectionToLocal(getParent(hydraulic.node), bx-ax, by-ay, bz-az);
						setDirection(hydraulic.node, x, y, z, 0, 1, 0);
						local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
						if hydraulic.isScaleAble then
							local xScale, yScale, zScale = getScale(hydraulic.punch);
							local newScale = yScale * (distance / hydraulic.punchDistance);
							setScale(hydraulic.punch, 1, 1, newScale);
						else
							if hydraulic.punch ~= nil then
								setTranslation(hydraulic.punch, 0, 0, distance-hydraulic.punchDistance);
							end;
						end;
					end;
				end;
			else
				local ax, ay, az = getWorldTranslation(hydraulic.node);
				local bx, by, bz = getWorldTranslation(hydraulic.FixPoint);
				local x, y, z = worldDirectionToLocal(getParent(hydraulic.node), bx-ax, by-ay, bz-az);
				setDirection(hydraulic.node, x, y, z, 0, 1, 0);
				local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
				if hydraulic.isScaleAble then
					local xScale, yScale, zScale = getScale(hydraulic.punch);
					local newScale = yScale * (distance / hydraulic.punchDistance);
					setScale(hydraulic.punch, 1, 1, newScale);
				else
					if hydraulic.punch ~= nil then
						setTranslation(hydraulic.punch, 0, 0, distance-hydraulic.punchDistance);
					end;
				end;
			end;	
		end;		
	end;
end;

function DF7807:draw()
    if not self.infoHUDactive then
    	g_currentMission:addHelpButtonText(g_i18n:getText("DF7807_HELP_ON"), InputBinding.DF7807_INFOHUD);
    else 	
    	self.infoHUDOverlay:render();
    	local infoText={};		
		if self.Go.FrontWindow then
			infoText[1] = g_i18n:getText("Key_Eng_Deu").." "..InputBinding.getKeyNamesOfDigitalAction(InputBinding.DF7807_WINDOW_1).." "..g_i18n:getText("DF7807_WINDOW_1_ON");
		else
			infoText[1] = g_i18n:getText("Key_Eng_Deu").." "..InputBinding.getKeyNamesOfDigitalAction(InputBinding.DF7807_WINDOW_1).." "..g_i18n:getText("DF7807_WINDOW_1_OFF");
		end;
		if self.Go.rearWindow then
			infoText[2] = g_i18n:getText("Key_Eng_Deu").." "..InputBinding.getKeyNamesOfDigitalAction(InputBinding.DF7807_WINDOW_2).." "..g_i18n:getText("DF7807_WINDOW_2_ON");
		else
			infoText[2] = g_i18n:getText("Key_Eng_Deu").." "..InputBinding.getKeyNamesOfDigitalAction(InputBinding.DF7807_WINDOW_2).." "..g_i18n:getText("DF7807_WINDOW_2_OFF");
		end;
		if self.Go.RoofWindow then
			infoText[3] = g_i18n:getText("Key_Eng_Deu").." "..InputBinding.getKeyNamesOfDigitalAction(InputBinding.DF7807_WINDOW_3).." "..g_i18n:getText("DF7807_WINDOW_3_ON");
		else
			infoText[3] = g_i18n:getText("Key_Eng_Deu").." "..InputBinding.getKeyNamesOfDigitalAction(InputBinding.DF7807_WINDOW_3).." "..g_i18n:getText("DF7807_WINDOW_3_OFF");
		end;			
		if self.WorklightsActive then
			infoText[4] = g_i18n:getText("Key_Eng_Deu").." "..InputBinding.getKeyNamesOfDigitalAction(InputBinding.DF7807_WORKLIGHTS).." "..g_i18n:getText("DF7807_WORKLIGHTS_OFF");
		else
			infoText[4] = g_i18n:getText("Key_Eng_Deu").." "..InputBinding.getKeyNamesOfDigitalAction(InputBinding.DF7807_WORKLIGHTS).." "..g_i18n:getText("DF7807_WORKLIGHTS_ON");
		end;
		if self.FenderActive then
			infoText[5] = g_i18n:getText("Key_Eng_Deu").." "..InputBinding.getKeyNamesOfDigitalAction(InputBinding.DF7807_FENDER).." "..g_i18n:getText("DF7807_FENDER_OFF");
		else
			infoText[5] = g_i18n:getText("Key_Eng_Deu").." "..InputBinding.getKeyNamesOfDigitalAction(InputBinding.DF7807_FENDER).." "..g_i18n:getText("DF7807_FENDER_ON");
		end;		
		infoText[6] = g_i18n:getText("Key_Eng_Deu").." "..InputBinding.getKeyNamesOfDigitalAction(InputBinding.DF7807_WHEELS).." "..g_i18n:getText("DF7807_WHEELS");
		infoText[7] = g_i18n:getText("Key_Eng_Deu").." "..InputBinding.getKeyNamesOfDigitalAction(InputBinding.DF7807_INFOHUD).." "..g_i18n:getText("DF7807_HELP_OFF");
    	for i=1, table.getn(infoText) do
    		renderText((self.infoHUDPoxX+0.015),((self.infoHUDPoxY+self.infoHUDHeight) - (0.03*i)),0.026,infoText[i]);
    	end;
    end;
end;

function DF7807:onLeave()
	self.WorklightsActive = false;
	for i=1, table.getn(self.Worklights) do
		setVisibility(self.Worklights[i], self.WorklightsActive);
	end;
end;